package com.topsunit.query.binding;

import com.topsunit.query.annotations.Param;

import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.Arrays;
import java.util.Collection;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author yonghuan
 */
public class DefaultSqlTranslator implements SqlTranslator {

    private final Map<String, String> sqlCache = new ConcurrentHashMap<>(200);

    @Override
    public String translate(String sql, Method method, Object[] args) {
        final String cacheKey = method.getDeclaringClass().getName() + "." + method.getName();
        String cachedSql = sqlCache.get(cacheKey);
        if (cachedSql != null) {
            return cachedSql;
        }
        sql = translateInternal(sql, method, args);
        sqlCache.put(cacheKey, sql);
        return sql;
    }

    private String translateInternal(String sql, Method method, Object[] args) {
        if (useNamedParameter(method, args)) {
            Parameter[] parameters = method.getParameters();
            for (int i = 0; i < parameters.length; i++) {
                Parameter parameter = parameters[i];
                Param param = parameter.getAnnotation(Param.class);
                String paramName = param.value();
                Object paramValue = args[i];
                if (paramValue instanceof Collection) {
                    String str = (String) ((Collection) paramValue).stream()
                            .map(obj -> "?")
                            .collect(Collectors.joining(","));
                    sql = sql.replaceAll(":" + paramName, str);
                } else {
                    sql = sql.replaceAll(":" + paramName, "?");
                }
            }
        }
        return sql;
    }

    private boolean useNamedParameter(Method method, Object[] args) {
        long paramAnnotationCount = Arrays.stream(method.getParameters())
                .map(parameter -> parameter.getAnnotation(Param.class))
                .filter(Objects::nonNull)
                .count();
        if (paramAnnotationCount == 0) {
            return false;
        }
        if (paramAnnotationCount != args.length) {
            throw new IllegalArgumentException("Param注解数量与实参数量不一致");
        }
        return true;
    }

}
